"""Suite de test para clase principal."""
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
import platform
from unittest.mock import Mock, patch

import pandas as pd
import pytest
from azure.identity import AuthenticationRecord
from azure.identity.aio import DefaultAzureCredential
from azure.core.exceptions import ResourceNotFoundError
from adlfs import AzureBlobFileSystem

from azure_datalake_utils import Datalake
from azure_datalake_utils.exepctions import ExtensionIncorrecta

fake_record = AuthenticationRecord("tenant-id", "client-id", "localhost", "object.tenant", "username")


@pytest.fixture
def dl_account() -> Datalake:
    """DL instancia incilizada."""
    return Datalake.from_account_key('name', 'key')


@pytest.fixture
def test_df() -> pd.DataFrame:
    """DF para test."""
    return pd.DataFrame({"foo_id": [1, 2, 3]})


@pytest.fixture
def test_str_df() -> pd.DataFrame:
    """DF para test."""
    return pd.DataFrame({"foo_str": ['bar\n', 'foo,', 'bar|'], "bar_str": ["bar", "foo|", "bar\r"], "foo": [1, 2, 3]})


def test_datalake_should_init_properly():
    """Test de inicializacion de Datalake."""
    with patch('azure.identity.InteractiveBrowserCredential.authenticate', return_value=fake_record):
        dl = Datalake('name', 'tenant')
        if platform.system().lower() != 'windows':
            assert dl.storage_options == {'account_name': 'name', 'anon': False}
        else:
            assert dl.storage_options['account_name'] == 'name'
            assert not dl.storage_options['anon']
            assert isinstance(dl.storage_options['credential'], DefaultAzureCredential)

    with patch('azure.identity.InteractiveBrowserCredential.authenticate', return_value=fake_record):
        dl = Datalake('name', 'tenant', fsspec_cache=False)
        if platform.system().lower() != 'windows':
            assert dl.storage_options == {
                'account_name': 'name',
                'anon': False,
                'default_cache_type': None,
                'default_fill_cache': False,
                'skip_instance_cache': True,
            }
        else:
            assert dl.storage_options['account_name'] == 'name'
            assert dl.storage_options['default_cache_type'] is None, "no se esta invalidando el cache."
            assert not dl.storage_options['anon']
            assert isinstance(dl.storage_options['credential'], DefaultAzureCredential)


def test_datalake_should_init_from_account_key():
    """Test para inicializacion del datalake con key account."""
    dl = Datalake.from_account_key('name', 'key')
    assert dl.storage_options == {'account_name': 'name', 'account_key': 'key'}

    dl = Datalake.from_account_key('name', 'key', False)
    assert dl.storage_options == {
        'account_name': 'name',
        'account_key': 'key',
        'default_cache_type': None,
        'default_fill_cache': False,
        'skip_instance_cache': True,
    }, "no se esta invalidando el cache."


def test_verificar_extension_should_check_extensions():
    """Test de verificar de Datalake."""
    with patch('azure.identity.InteractiveBrowserCredential.authenticate', return_value=fake_record):
        dl = Datalake('name', 'tenant')
        assert dl._verificar_extension('a/b/c/foo.csv', '.csv', '.tsv')
        assert dl._verificar_extension('a/b/c/foo.tsv', '.csv', '.tsv', '.txt')

        with pytest.raises(ExtensionIncorrecta):
            dl._verificar_extension('a/b/c/foo.xlsx', '.csv', '.tsv', '.txt')


def test_read_csv_should_raise_extension_incorrecta(dl_account: Datalake):
    """Test de verificar raise de ExtensionIncorrecta."""
    with pytest.raises(ExtensionIncorrecta):
        dl_account.read_csv('contenedor/foo/bar.text')


@patch("azure_datalake_utils.azure_datalake_utils.pd.read_csv")
def test_read_csv_should_return_dataframe(read_mock: Mock, dl_account: Datalake, test_df: pd.DataFrame):
    """Test read_csv."""
    read_mock.return_value = test_df
    df = dl_account.read_csv("path/to/file.csv")
    read_mock.assert_called_once()
    pd.testing.assert_frame_equal(df, test_df)


@patch("azure_datalake_utils.azure_datalake_utils.pd.read_csv")
def test_read_csv_with_list_of_path_should_return_dataframe(
    read_mock: Mock, dl_account: Datalake, test_df: pd.DataFrame
):
    """Test read_csv."""
    read_mock.return_value = test_df
    files = ["path/to/file.csv", "path/to/file.csv"]
    df = dl_account.read_csv(files)
    assert read_mock.call_count == len(files)
    pd.testing.assert_frame_equal(df, pd.concat([test_df, test_df], ignore_index=True))


@patch("azure_datalake_utils.azure_datalake_utils.pd.read_json")
def test_read_json_should_return_dataframe(read_mock: Mock, dl_account: Datalake, test_df: pd.DataFrame):
    """Test read_json."""
    read_mock.return_value = test_df
    df = dl_account.read_json("path/to/file.json")
    read_mock.assert_called_once()
    pd.testing.assert_frame_equal(df, test_df)


@patch("azure_datalake_utils.azure_datalake_utils.pd.read_excel")
def test_read_excel_should_return_dataframe(read_mock: Mock, dl_account: Datalake, test_df: pd.DataFrame):
    """Test read_json."""
    read_mock.return_value = test_df
    df = dl_account.read_excel("path/to/file.xlsx")
    read_mock.assert_called_once()
    pd.testing.assert_frame_equal(df, test_df)


def test_limpiar_df_cols_str_should_clean_no_special_chars(dl_account: Datalake, test_str_df: pd.DataFrame):
    """Test para limpiar el DF."""
    # valores originales:
    # "foo_str": ['bar\n', 'foo,', 'bar|']
    # "bar_str": ["bar", "foo|", "bar\r"]
    # Test con `,`
    df_clean = dl_account._limpiar_df_cols_str(test_str_df, ',')
    assert df_clean['foo_str'].values.tolist() == ['bar ', 'foo ', 'bar|']
    assert df_clean['bar_str'].values.tolist() == ['bar', 'foo|', 'bar ']
    assert df_clean['foo'].values.tolist() == [1, 2, 3]


def test_limpiar_df_cols_str_should_clean_special_chars(dl_account: Datalake, test_str_df: pd.DataFrame):
    """Test para limpiar el DF."""
    # Test con `|`
    df_clean = dl_account._limpiar_df_cols_str(test_str_df, '|')
    assert df_clean['foo_str'].values.tolist() == ['bar ', 'foo,', 'bar ']
    assert df_clean['bar_str'].values.tolist() == ['bar', 'foo ', 'bar ']
    assert df_clean['foo'].values.tolist() == [1, 2, 3]


@patch("azure_datalake_utils.azure_datalake_utils.pd.read_csv", autospec=True)
def test_read_excel_should_raise_ArchivNoEncontrado(read_mock: Mock, dl_account: Datalake):
    """Test para verificar que la expecion es levantanda."""
    read_mock.side_effect = FileNotFoundError
    with pytest.raises(ExtensionIncorrecta):
        dl_account.read_csv('contenedor/foo/bar.text')


@patch("azure_datalake_utils.azure_datalake_utils.pd.read_csv", autospec=True)
def test_read_excel_should_raise_ArchivNoEncontrado_with_azure_error(read_mock: Mock, dl_account: Datalake):
    """Test para verificar que la expecion es levantanda."""
    read_mock.side_effect = ResourceNotFoundError
    with pytest.raises(ExtensionIncorrecta):
        dl_account.read_csv('contenedor/foo/bar.text')


@patch("azure_datalake_utils.azure_datalake_utils.pd.read_csv")
@patch("azure_datalake_utils.azure_datalake_utils.AzureBlobFileSystem", autospec=True)
@patch("azure_datalake_utils.partitions.HivePartitiion.get_partition_list")
def test_read_csv_with_partition_should_return_df_read_from_partitions(
    get_partition_list_mock: Mock,
    fs_mock: AzureBlobFileSystem,
    read_mock,
    dl_account: Datalake,
    test_df: pd.DataFrame,
):
    """Test para read_csv_with_partition."""
    get_partition_list_mock.return_value = [
        'contenedor/file/path/part=1/file.csv',
        'contenedor/file/path/part=2/file.csv',
    ]
    read_mock.return_value = test_df
    dl_account.fs = fs_mock
    with patch(
        "azure_datalake_utils.azure_datalake_utils.HivePartitiion.get_partition_files",
        return_value=[('file.csv', {'part': '1'}), ('file.csv', {'part': '2'})],
        create=True,
    ):
        df = dl_account.read_csv_with_partition("contenedor/file/path/")

    assert read_mock.call_count == 2
    assert df['part'].to_list() == ['1', '1', '1', '2', '2', '2']



"""Tests for `azure_datalake_utils.partitions` package."""
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
import datetime

import pytest

from unittest.mock import patch
from typing import List, Dict
from azure_datalake_utils.partitions import HivePartitiion


def list_side_effect_to_test(*args) -> List[Dict]:
    """Funcion para controlar los side effects de fs.

    Simula una estructura de archivos de la siguiente manera:
    - contenedor/ruta/al/archivo/year=2022/month=10/
        archivo.csv
    - contenedor/ruta/al/archivo/year=2023/month=10/
        archivo.csv
    - contenedor/ruta/al/archivo/year=2022/month=11/
        archivo.csv
    - contenedor/ruta/al/archivo/year=2022/month=1/
        archivo1.csv
        archivo2.csv

    """
    if args[0] == "contenedor/ruta/al/archivo/year=2022/month=10/":
        return [{"name": "contenedor/ruta/al/archivo/year=2022/month=10/archivo.csv"}]
    elif args[0] == "contenedor/ruta/al/archivo/year=2023/month=10/":
        return [{"name": "contenedor/ruta/al/archivo/year=2023/month=10/archivo.csv"}]
    elif args[0] == "contenedor/ruta/al/archivo/year=2022/month=11/":
        return [{"name": "contenedor/ruta/al/archivo/year=2022/month=11/archivo.csv"}]
    elif args[0] == "contenedor/ruta/al/archivo/year=2022/month=11/":
        return []
    elif args[0] == "contenedor/ruta/al/archivo/year=2022/month=1/":
        return [
            {"name": "contenedor/ruta/al/archivo/year=2022/month=1/archivo1.csv"},
            {"name": "contenedor/ruta/al/archivo/year=2022/month=1/archivo2.csv"},
        ]
    else:
        raise FileNotFoundError


def list_side_effect_to_test_last_level(*args, **kwargs) -> Dict:
    """Funcion para controlar los side effects de fs.

    Simula una estructura de archivos de la siguiente manera:
    - contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-01/archivo.csv
    - contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-02/archivo.csv
    - contenedor/ruta/al/archivo/year=2022/month=11/load_date=2022-01-01/archivo.csv

    """
    if args[0] == "contenedor/ruta/al/archivo/year=2022/month=10/":
        return {
            'contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-01/archivo.csv': {
                'last_modified': datetime.datetime(2022, 1, 1, 12, 30, 00, tzinfo=datetime.timezone.utc)
            },
            'contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-02/archivo.csv': {
                'last_modified': datetime.datetime(2022, 1, 2, 12, 30, 00, tzinfo=datetime.timezone.utc)
            },
        }
    elif args[0] == "contenedor/ruta/al/archivo/year=2022/month=11/":
        return {
            'contenedor/ruta/al/archivo/year=2022/month=11/load_date=2022-01-01/archivo.csv': {
                'last_modified': datetime.datetime(2022, 1, 1, 12, 30, 00, tzinfo=datetime.timezone.utc)
            },
        }
    else:
        return {}


@patch("azure_datalake_utils.partitions.AzureBlobFileSystem", autospec=True)
def test__make_partitions_using_partition_cols_no_filter_deeper_level_return_correct_list(fs_mock):
    """Test para verificar discover."""
    fs_mock.listdir.side_effect = list_side_effect_to_test
    hive = HivePartitiion(
        ruta="contenedor/ruta/al/archivo/", partition_cols={'year': ['2022', '2023'], 'month': ['10']}, fs=fs_mock
    )
    assert hive.partition_files == [
        ("archivo.csv", {'year': '2022', 'month': '10'}),
        ("archivo.csv", {'year': '2023', 'month': '10'}),
    ]

    assert hive.container == "contenedor"
    assert hive.path_blob == "ruta/al/archivo/"


@patch("azure_datalake_utils.partitions.AzureBlobFileSystem", autospec=True)
def test__make_partitions_using_partition_cols_and_partition_exclusion_should_return_filtered_partition(fs_mock):
    """Test para verificar discover con exlcusion de particiones."""
    fs_mock.listdir.side_effect = list_side_effect_to_test
    hive = HivePartitiion(
        ruta="contenedor/ruta/al/archivo/",
        partition_cols={'year': ['2022'], 'month': ['10', '11', '1']},
        fs=fs_mock,
        partition_exclusion={'month': ['11', '1']},
    )
    assert hive.partition_files == [
        ("archivo.csv", {'year': '2022', 'month': '10'}),
    ]


@patch("azure_datalake_utils.partitions.AzureBlobFileSystem", autospec=True)
def test__make_partitions_using_partition_cols_and_partition_exclusion_should_rasie_expection(fs_mock):
    """Test para verificar discover con exlcusion de particiones."""
    fs_mock.listdir.side_effect = list_side_effect_to_test

    with pytest.raises(KeyError):

        HivePartitiion(
            ruta="contenedor/ruta/al/archivo/",
            partition_cols={'year': ['2022'], 'month': ['10', '11', '1']},
            fs=fs_mock,
            partition_exclusion={'month': ['11', '1'], 'product': ['foo']},
        )


@patch("azure_datalake_utils.partitions.AzureBlobFileSystem", autospec=True)
def test__make_partitions_using_partition_cols_no_last_modified_last_level_return_valid_files(fs_mock):
    """Test para verificar discover."""
    fs_mock.listdir.side_effect = list_side_effect_to_test
    hive = HivePartitiion(
        ruta="contenedor/ruta/al/archivo/", partition_cols={'year': ['2022'], 'month': ['10', '11', '1']}, fs=fs_mock
    )
    assert hive.partition_files == [
        ("archivo.csv", {'year': '2022', 'month': '10'}),
        ("archivo.csv", {'year': '2022', 'month': '11'}),
        ("archivo1.csv", {'year': '2022', 'month': '1'}),
    ]


@patch("azure_datalake_utils.partitions.AzureBlobFileSystem", autospec=True)
def test__make_partitions_using_partition_cols_no_last_modified_last_level_return_existing_files(fs_mock):
    """Test para verificar discover."""
    fs_mock.listdir.side_effect = list_side_effect_to_test
    hive = HivePartitiion(
        ruta="contenedor/ruta/al/archivo/", partition_cols={'year': ['2022', '2024'], 'month': ['10', '11']}, fs=fs_mock
    )
    assert hive.partition_files == [
        ("archivo.csv", {'year': '2022', 'month': '10'}),
        ("archivo.csv", {'year': '2022', 'month': '11'}),
    ]


@patch("azure_datalake_utils.partitions.AzureBlobFileSystem", autospec=True)
def test__make_partitions_using_partition_cols_filter_last_modified_last_level_return_valid_files(fs_mock):
    """Test para verificar discover."""
    fs_mock.find.side_effect = list_side_effect_to_test_last_level

    hive = HivePartitiion(
        ruta="contenedor/ruta/al/archivo/",
        partition_cols={'year': ['2022'], 'month': ['10', '11']},
        fs=fs_mock,
        last_modified_last_level=True,
    )

    assert hive.partition_files == [
        ("load_date=2022-01-02/archivo.csv", {'year': '2022', 'month': '10'}),
        ("load_date=2022-01-01/archivo.csv", {'year': '2022', 'month': '11'}),
    ]


@patch("azure_datalake_utils.partitions.AzureBlobFileSystem", autospec=True)
def test___discover_should_discover_partitions(fs_mock):
    """Test para verificar discover."""
    fs_mock.find.return_value = {
        'contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-01/archivo.csv': {
            'last_modified': datetime.datetime(2022, 1, 1, 12, 30, 00, tzinfo=datetime.timezone.utc)
        },
        'contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-02/archivo.csv': {
            'last_modified': datetime.datetime(2022, 1, 2, 12, 30, 00, tzinfo=datetime.timezone.utc)
        },
        'contenedor/ruta/al/archivo/year=2022/month=11/load_date=2022-01-01/archivo.csv': {
            'last_modified': datetime.datetime(2022, 1, 1, 12, 30, 00, tzinfo=datetime.timezone.utc)
        },
    }

    hive = HivePartitiion(
        ruta="contenedor/ruta/al/archivo/",
        fs=fs_mock,
        last_modified_last_level=False,
    )

    assert hive.partition_files == [
        ("archivo.csv", {'year': '2022', 'month': '10', 'load_date': '2022-01-02'}),
        ("archivo.csv", {'year': '2022', 'month': '10', 'load_date': '2022-01-01'}),
        ("archivo.csv", {'year': '2022', 'month': '11', 'load_date': '2022-01-01'}),
    ]


@patch("azure_datalake_utils.partitions.AzureBlobFileSystem", autospec=True)
def test___discover_with_filter_last_modified_last_level_should_discover_partitions(fs_mock):
    """Test para verificar discover."""
    fs_mock.find.return_value = {
        'contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-01/archivo.csv': {
            'last_modified': datetime.datetime(2022, 1, 1, 12, 30, 00, tzinfo=datetime.timezone.utc)
        },
        'contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-02/archivo.csv': {
            'last_modified': datetime.datetime(2022, 1, 2, 12, 30, 00, tzinfo=datetime.timezone.utc)
        },
        'contenedor/ruta/al/archivo/year=2022/month=11/load_date=2022-01-01/archivo.csv': {
            'last_modified': datetime.datetime(2022, 1, 1, 12, 30, 00, tzinfo=datetime.timezone.utc)
        },
    }

    hive = HivePartitiion(
        ruta="contenedor/ruta/al/archivo/",
        fs=fs_mock,
        last_modified_last_level=True,
    )

    assert hive.partition_files == [
        ("load_date=2022-01-02/archivo.csv", {'year': '2022', 'month': '10'}),
        ("load_date=2022-01-01/archivo.csv", {'year': '2022', 'month': '11'}),
    ]


@patch("azure_datalake_utils.partitions.AzureBlobFileSystem", autospec=True)
def test___discover_with_filter_last_modified_last_level_and_partition_exclusion_should_discover_partitions(fs_mock):
    """Test para verificar discover."""
    fs_mock.find.return_value = {
        'contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-01/archivo.csv': {
            'last_modified': datetime.datetime(2022, 1, 1, 12, 30, 00, tzinfo=datetime.timezone.utc)
        },
        'contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-02/archivo.csv': {
            'last_modified': datetime.datetime(2022, 1, 2, 12, 30, 00, tzinfo=datetime.timezone.utc)
        },
        'contenedor/ruta/al/archivo/year=2022/month=11/load_date=2022-01-01/archivo.csv': {
            'last_modified': datetime.datetime(2022, 1, 1, 12, 30, 00, tzinfo=datetime.timezone.utc)
        },
    }

    hive = HivePartitiion(
        ruta="contenedor/ruta/al/archivo/",
        fs=fs_mock,
        last_modified_last_level=True,
        partition_exclusion={'month': ['10']},
    )

    assert hive.partition_files == [
        ("load_date=2022-01-01/archivo.csv", {'year': '2022', 'month': '11'}),
    ]


@patch("azure_datalake_utils.partitions.AzureBlobFileSystem", autospec=True)
def test___discover_with_filter_last_modified_last_level_and_partition_inclusion_should_discover_partitions(fs_mock):
    """Test para verificar discover."""
    fs_mock.find.return_value = {
        'contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-01/archivo.csv': {
            'last_modified': datetime.datetime(2022, 1, 1, 12, 30, 00, tzinfo=datetime.timezone.utc)
        },
        'contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-02/archivo.csv': {
            'last_modified': datetime.datetime(2022, 1, 2, 12, 30, 00, tzinfo=datetime.timezone.utc)
        },
        'contenedor/ruta/al/archivo/year=2022/month=11/load_date=2022-01-01/archivo.csv': {
            'last_modified': datetime.datetime(2022, 1, 1, 12, 30, 00, tzinfo=datetime.timezone.utc)
        },
    }

    hive = HivePartitiion(
        ruta="contenedor/ruta/al/archivo/",
        fs=fs_mock,
        last_modified_last_level=True,
        partition_inclusion={'month': ['10']},
    )

    assert hive.partition_files == [
        ("load_date=2022-01-02/archivo.csv", {'year': '2022', 'month': '10'}),
    ]


@patch("azure_datalake_utils.partitions.AzureBlobFileSystem", autospec=True)
def test_get_partitiion_list_should_return_list_files(fs_mock):
    """Test para verificar get_partition_list."""
    fs_mock.listdir.side_effect = list_side_effect_to_test
    hive = HivePartitiion(
        ruta="contenedor/ruta/al/archivo/", partition_cols={'year': ['2022', '2024'], 'month': ['10', '11']}, fs=fs_mock
    )

    partition_list = hive.get_partition_list()

    assert partition_list == [
        'contenedor/ruta/al/archivo/year=2022/month=10/archivo.csv',
        'contenedor/ruta/al/archivo/year=2022/month=11/archivo.csv',
    ]



"""Main module."""
import platform
import re
from typing import Any, Dict, Optional, Union, List

import numpy as np
import pandas as pd
import azure_datalake_utils.experimental as exp

from azure.identity import InteractiveBrowserCredential
from azure.identity.aio import DefaultAzureCredential as AIODefaultAzureCredential
from adlfs import AzureBlobFileSystem
from azure_datalake_utils.exepctions import ExtensionIncorrecta, raiseArchivoNoEncontrado
from azure_datalake_utils.partitions import HivePartitiion


class Datalake(object):
    """Clase para representar operaciones de Datalake."""

    def __init__(
        self, datalake_name: str, tenant_id: str, account_key: Optional[str] = None, fsspec_cache: bool = True
    ) -> None:
        """Clase para interactuar con Azure Dalake.

        Args:
            datalake_name: nombre de la cuenta de Azure Datalake Gen2.
            tenant_id: Identificador del tenant, es valor es proporcionado
                por arquitectura de datos, debe conservarse para un
                correcto funcionamiento.
            account_key: key de la cuenta. Por defecto es None y es ignorado.
            fsspec_cache: indica si se va usar la funcionalidad de cache de fsspec.
                **NOTA**: se recomienda usar `fsspec_cache=False` cuando se encuentran en ambientes
                serverless, donde no se tiene control escricto de la `vida` de la instancias, de esta manera
                la libreria siempre verifica la información con Azure y no reusar instancias en cache que pueden
                tener valores desactualizados.

                Para ver los efectos, ver los siguientes issues:
                - https://github.com/fsspec/adlfs/issues/391
                - https://github.com/Azure/azure-sdk-for-python/issues/28312
        """
        self.datalake_name = datalake_name

        if account_key is None:
            self.tenant_id = tenant_id
            credentials = InteractiveBrowserCredential(tenant_id=self.tenant_id)
            credentials.authenticate()
            self._credentials = credentials
            # TODO: verificar https://github.com/fsspec/adlfs/issues/270
            # para ver como evoluciona y evitar este condicional.
            if platform.system().lower() != 'windows':
                storage_options = {'account_name': self.datalake_name, 'anon': False}
            else:
                storage_options = {
                    'account_name': self.datalake_name,
                    'anon': False,
                    'credential': AIODefaultAzureCredential(),
                }

        else:
            storage_options = {'account_name': self.datalake_name, 'account_key': account_key}
            self.fs = AzureBlobFileSystem(account_name=self.datalake_name, account_key=account_key)

        if not fsspec_cache:
            storage_options["default_cache_type"] = None
            storage_options["default_fill_cache"] = False
            storage_options["skip_instance_cache"] = True

        self.storage_options = storage_options

    @classmethod
    def from_account_key(cls, datalake_name: str, account_key: str, fsspec_cache: bool = True):
        """Opcion de inicializar con account key."""
        return cls(datalake_name=datalake_name, account_key=account_key, tenant_id=None, fsspec_cache=fsspec_cache)

    @raiseArchivoNoEncontrado
    def read_csv(self, ruta: Union[str, List[str]], **kwargs: Optional[Any]) -> pd.DataFrame:
        """Leer un archivo CSV desde la cuenta de datalake.

        Esta función hace una envoltura de [pd.read_csv].
        usar la documentación de la función para determinar parametros adicionales.

        [pd.read_csv]: https://pandas.pydata.org/docs/reference/api/pandas.read_csv.html

        Args:

            ruta: Ruta a leeder el archivo, debe contener una referencia a un archivo
                `.csv` o `.txt`. Recordar que la ruta debe contener esta estructura:
                `{NOMBRE_CONTENEDOR}/{RUTA}/{nombre o patron}.csv`.
                NUEVO en version 0.5:
                Tambien acepta una lista de archivos terminados en csv. ejemplo:
                ```
                [{NOMBRE_CONTENEDOR}/{RUTA}/{nombre o patron}.csv,
                {NOMBRE_CONTENEDOR}/{RUTA2}/{nombre o patron}.csv]
                ```

            **kwargs: argumentos a pasar a pd.read_csv. El unico argumento que es ignorado
                es storage_options.

        Returns:
            Dataframe con la informacion del la ruta.
        """
        if 'storage_options' in kwargs:
            kwargs.pop('storage_options')

        if type(ruta) == str:
            self._verificar_extension(ruta, '.csv', '.txt', '.tsv')
            df = pd.read_csv(f"az://{ruta}", storage_options=self.storage_options, **kwargs)
        else:
            [self._verificar_extension(r, '.csv', '.txt', '.tsv') for r in ruta]
            rutas = [pd.read_csv(f"az://{r}", storage_options=self.storage_options, **kwargs) for r in ruta]
            df = pd.concat(rutas, ignore_index=True)

        return df

    @raiseArchivoNoEncontrado
    def read_excel(self, ruta: str, experimental: bool = False, **kwargs: Optional[Any]) -> pd.DataFrame:
        """Leer un archivo Excel desde la cuenta de datalake.

        Esta función hace una envoltura de [pd.read_excel].
        Por favor usar la documentación de la función para determinar parametros adicionales.

        [[pd.read_excel]]:(https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_excel.html)

        Args:
            ruta: Ruta a leeder el archivo, debe contener una referencia a un archivo
                `.xlsx` o `.xls`. Recordar que la ruta debe contener esta estructura:
                `{NOMBRE_CONTENEDOR}/{RUTA}/{nombre o patron}.xlsx`.
            force_client: Bandera para forzar el uso del cliente. Es flag todavia es experimental
                Y en futuras versiones se va ha eliminar. Solo funciona con un solo archivo de excel.
            **kwargs: argumentos a pasar a pd.read_excel.


        Returns:
            Dataframe con la informacion del la ruta.
        """
        if 'storage_options' in kwargs:
            kwargs.pop('storage_options')

        if 'engine' in kwargs:
            kwargs.pop('engine')

        self._verificar_extension(ruta, '.xlsx', '.xls')

        # TODO: esto es algo temporal y se debe analizar si se puede remover. Esta bandera fue necesario debido a:
        # 1. Si no se usa el cliente para descargar el excel cuando se modifica el archivo de
        # excel y no se ha reiniciado
        # el runtime se provoca el error: `BadZipFile("File is not a zip file")`.
        # 2. Se debe analizar como integrar en windows donde se usa credenciales asociadas al Active directory.
        if experimental:
            df = exp.read_excel_with_client(ruta, self.datalake_name, self.storage_options['account_key'], **kwargs)
            return df
        ###############

        df = pd.read_excel(f"az://{ruta}", engine='openpyxl', storage_options=self.storage_options, **kwargs)

        return df

    @raiseArchivoNoEncontrado
    def read_json(self, ruta: str, **kwargs: Optional[Any]) -> pd.DataFrame:
        """Leer un archivo Json desde la cuenta de datalake.

        Esta función hace una envoltura de [pd.read_json].
        Por favor usar la documentación de la función para determinar parametros adicionales.

        [[pd.read_json]]:(https://pandas.pydata.org/docs/reference/api/pandas.read_json.html)

        Args:
            ruta: Ruta a leeder el archivo, debe contener una referencia a un archivo
                `.json` . Recordar que la ruta debe contener esta estructura:
                `{NOMBRE_CONTENEDOR}/{RUTA}/{nombre o patron}.json`.
            **kwargs: argumentos a pasar a pd.read_json.


        Returns:
            Dataframe con la informacion del la ruta.
        """
        if 'storage_options' in kwargs:
            kwargs.pop('storage_options')

        self._verificar_extension(ruta, '.json')

        df = pd.read_json(f"az://{ruta}", storage_options=self.storage_options, **kwargs)

        return df

    def read_csv_with_partition(
        self,
        ruta: str,
        partition_cols: Dict[str, List[str]] = None,
        partition_exclusion: Dict[str, List[str]] = None,
        partition_inclusion: Dict[str, List[str]] = None,
        last_modified_last_level: bool = True,
        **kwargs: Optional[Any],
    ) -> pd.DataFrame:
        """Leer un archivo CSV desde la cuenta de datalake con particiones Hive.

        Una partición tipo Hive son archivos almacenados de la forma
        /ruta/to/archivo/particion_1=1/particion2=2/archivo_con_info.extension.

        **IMPORTANTE**: por el momento el funcionamiento de esta función es solo soportada
        si el objeto fue creado mediante `from_account_key`.

        Esta función hace una envoltura de `read_csv` que asu vez usa [pd.read_csv].
        usar la documentación de la función para determinar parametros adicionales.

        [pd.read_csv]: https://pandas.pydata.org/docs/reference/api/pandas.read_csv.html

        Args:

            ruta: Ruta a leeder el archivo, esta ruta debe contener archivos guardados siguiendo
            la convección de la partición Hive. Ejemplo si la estructura de los archivos es:

            - contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-01/archivo.csv
            - contenedor/ruta/al/archivo/year=2022/month=10/load_date=2022-01-02/archivo.csv
            - contenedor/ruta/al/archivo/year=2022/month=11/load_date=2022-01-01/archivo.csv

            `ruta` debe ser `contenedor/ruta/al/archivo/`.

            partition_cols: Definir que particionews incluir, se asume que se sabe a priori la
                estructura. Si se pasa `None`, se activa el descubrimiento automatico de las particiones.
            partition_exclusion: Definir que particiones excluir, es más util cuando no se ha definido
                `partition_cols=None`.
            partition_inclusion: Definir la particiones a incluir, es más util cuando no se ha definido
                `partition_cols=None`.
            last_modified_last_level: Define si el ultimo nivel no se tiene en cuenta para particiones y
                se carga el archivo que fue modificado de manera más reciente.
            **kwargs: argumentos a pasar a pd.read_csv. El unico argumento que es ignorado
                es storage_options.

        Returns:
            Dataframe con la informacion del la ruta.
        """
        if not ruta.endswith("/"):
            raise ValueError("ruta debe finalizar en /")

        particiones = HivePartitiion(
            ruta=ruta,
            partition_cols=partition_cols,
            partition_exclusion=partition_exclusion,
            partition_inclusion=partition_inclusion,
            last_modified_last_level=last_modified_last_level,
            fs=self.fs,
        )
        list_of_files = particiones.get_partition_list()
        list_of_dfs = []
        for path_, particion in zip(list_of_files, particiones.get_partition_files()):
            particiones_cols = particion[1]
            df = self.read_csv(path_, **kwargs).assign(**particiones_cols)
            list_of_dfs.append(df)

        return pd.concat(list_of_dfs, ignore_index=True)

    def write_csv(self, df: pd.DataFrame, ruta, **kwargs: Optional[Any]) -> None:
        """Escribir al archivo."""
        if not self._verificar_extension(ruta, '.csv', '.txt', '.tsv'):
            raise ExtensionIncorrecta(ruta)

        sep = kwargs.get('sep', ',')
        df_to_write = self._limpiar_df_cols_str(df, sep)
        df_to_write.to_csv(f"az://{ruta}", storage_options=self.storage_options, **kwargs)

    def write_excel(self, df: pd.DataFrame, ruta, **kwargs: Optional[Any]) -> None:
        """Escribir al archivo al datalake."""
        if not self._verificar_extension(ruta, '.xlsx', '.xls'):
            raise ExtensionIncorrecta(ruta)
        df.to_excel(f"az://{ruta}", storage_options=self.storage_options, **kwargs)

    def write_json(self, df: pd.DataFrame, ruta, **kwargs: Optional[Any]) -> None:
        """Escribir al archivo al datalake."""
        if not self._verificar_extension(ruta, '.json'):
            raise ExtensionIncorrecta(ruta)
        df.to_json(f"az://{ruta}", storage_options=self.storage_options, **kwargs)

    def _verificar_extension(self, ruta: str, *extensiones):
        """Metodo para verificar extensiones."""
        for ext in extensiones:
            verificar = ruta.endswith(ext)
            if verificar:
                return True
        raise ExtensionIncorrecta(ruta)

    def _limpiar_df_cols_str(self, df: pd.DataFrame, sep: str = ",") -> pd.DataFrame:
        """Limpia las columnas string del dataframe."""
        types = df.dtypes
        string_columns = list(types[types == np.array([object()]).dtype].index)
        esc_sep = re.escape(sep)
        df_res = df.copy()
        df_res[string_columns] = (
            df[string_columns]
            .replace(esc_sep, " ", regex=True)
            .replace("\r", " ", regex=True)
            .replace("\n", " ", regex=True)
        )
        return df_res
